import React, { useRef } from "react";
import {
  AddIcon,
  RemoveIcon,
  ResizeHeightIcon,
} from "../../../../../icons/ExportCommonIcons";
import RenameInput from "../../../../../ui/inputs/RenameInput";
import { handleResize } from "../../../../../../functions/handleResizePannel";
import {
  useSelectedAction,
  useSelectedEventData,
  useSelectedProduct,
} from "../../../../../../store/simulation/useSimulationStore";
import { MathUtils } from "three";
import { useProductStore } from "../../../../../../store/simulation/useProductStore";

interface ActionsListProps {
  setSelectedPointData: (data: any) => void; // You can replace `any` with a more specific type if you have one
  selectedPointData: any; // You can replace `any` with a more specific type if you have one
  // ui control props
  multipleAction?: boolean;
}

const ActionsList: React.FC<ActionsListProps> = ({
  setSelectedPointData,
  selectedPointData,
  multipleAction = false,
}) => {
  const actionsContainerRef = useRef<HTMLDivElement>(null);

  // store
  const { selectedEventData } = useSelectedEventData();
  const { updateAction, addAction, removeAction } = useProductStore();
  const { selectedProduct } = useSelectedProduct();
  const { selectedAction, setSelectedAction, clearSelectedAction } =
    useSelectedAction();

  const handleAddAction = () => {
    if (!selectedEventData || !selectedPointData) return;

    const newAction = {
      actionUuid: MathUtils.generateUUID(),
      actionName: `Action ${selectedPointData.actions.length + 1}`,
      actionType: "pickAndPlace" as const,
      process: {
        startPoint: null,
        endPoint: null,
      },
      triggers: [] as TriggerSchema[],
    };

    addAction(
      selectedProduct.productId,
      selectedEventData.data.modelUuid,
      selectedEventData.selectedPoint,
      newAction
    );

    const updatedPoint = {
      ...selectedPointData,
      actions: [...selectedPointData.actions, newAction],
    };
    setSelectedPointData(updatedPoint);
    setSelectedAction(newAction.actionUuid, newAction.actionName);
  };

  const handleDeleteAction = (actionUuid: string) => {
    if (!selectedPointData) return;

    removeAction(actionUuid);
    const newActions = selectedPointData.actions.filter(
      (a: any) => a.actionUuid !== actionUuid
    );

    const updatedPoint = {
      ...selectedPointData,
      actions: newActions,
    };
    setSelectedPointData(updatedPoint);

    if (selectedAction.actionId === actionUuid) {
      if (newActions.length > 0) {
        setSelectedAction(newActions[0].actionUuid, newActions[0].actionName);
      } else {
        clearSelectedAction();
      }
    }
  };

  const handleRenameAction = (newName: string) => {
    if (!selectedAction.actionId) return;
    updateAction(selectedAction.actionId, { actionName: newName });

    if (selectedPointData?.actions) {
      const updatedActions = selectedPointData.actions.map((action: any) =>
        action.actionUuid === selectedAction.actionId
          ? { ...action, actionName: newName }
          : action
      );
      setSelectedPointData({
        ...selectedPointData,
        actions: updatedActions,
      });
    } else {
      // write logic for single action
      return;
    }
  };

  const handleActionSelect = (actionUuid: string, actionName: string) => {
    setSelectedAction(actionUuid, actionName);
  };

  return (
    <div className="actions-list-container">
      <div className="actions">
        <div className="header">
          <div className="header-value">Actions</div>

          <button
            className="add-button"
            onClick={() => handleAddAction()}
            disabled={!multipleAction}
          >
            <AddIcon /> Add
          </button>
        </div>
        <div
          className="lists-main-container"
          ref={actionsContainerRef}
          style={{ height: "120px" }}
        >
          <div className="list-container">
            {multipleAction &&
              selectedPointData.actions.map((action: any) => (
                <div
                  key={action.actionUuid}
                  className={`list-item ${
                    selectedAction.actionId === action.actionUuid
                      ? "active"
                      : ""
                  }`}
                >
                  <button
                    className="value"
                    onClick={() =>
                      handleActionSelect(action.actionUuid, action.actionName)
                    }
                  >
                    <RenameInput
                      value={action.actionName}
                      onRename={handleRenameAction}
                    />
                  </button>
                  {selectedPointData.actions.length > 1 && (
                    <button
                      className="remove-button"
                      onClick={() => handleDeleteAction(action.actionUuid)}
                    >
                      <RemoveIcon />
                    </button>
                  )}
                </div>
              ))}
            {!multipleAction && selectedPointData && (
              <div
                key={selectedPointData.action.actionUuid}
                className={`list-item active`}
              >
                <button
                  className="value"
                  onClick={() =>
                    handleActionSelect(
                      selectedPointData.action.actionUuid,
                      selectedPointData.action.actionName
                    )
                  }
                >
                  <RenameInput
                    value={selectedPointData.action.actionName}
                    onRename={handleRenameAction}
                  />
                </button>
              </div>
            )}
          </div>
          {multipleAction && (
            <button
              className="resize-icon"
              id="action-resize"
              onMouseDown={(e: any) => handleResize(e, actionsContainerRef)}
            >
              <ResizeHeightIcon />
            </button>
          )}
        </div>
      </div>
    </div>
  );
};

export default ActionsList;
