import { useEffect, useState } from "react";
import InputWithDropDown from "../../../../../ui/inputs/InputWithDropDown";
import RenameInput from "../../../../../ui/inputs/RenameInput";
import LabledDropdown from "../../../../../ui/inputs/LabledDropdown";
import Trigger from "../trigger/Trigger";
import { useSelectedEventData, useSelectedProduct, useSelectedAction } from "../../../../../../store/simulation/useSimulationStore";
import { useProductStore } from "../../../../../../store/simulation/useProductStore";
import PickAndPlaceAction from "../actions/PickAndPlaceAction";
import ActionsList from "../components/ActionsList";

function RoboticArmMechanics() {
    const [activeOption, setActiveOption] = useState<"default" | "pickAndPlace">("default");
    const [selectedPointData, setSelectedPointData] = useState<RoboticArmPointSchema | undefined>();
    const { selectedEventData } = useSelectedEventData();
    const { getPointByUuid, updateEvent, updateAction } = useProductStore();
    const { selectedProduct } = useSelectedProduct();
    const { selectedAction, setSelectedAction, clearSelectedAction } = useSelectedAction();

    useEffect(() => {
        if (selectedEventData) {
            const point = getPointByUuid(
                selectedProduct.productId,
                selectedEventData.data.modelUuid,
                selectedEventData.selectedPoint
            ) as RoboticArmPointSchema | undefined;
            if (point?.actions) {
                setSelectedPointData(point);
                setActiveOption(point.actions[0].actionType as "default" | "pickAndPlace");
                if (point.actions.length > 0 && !selectedAction.actionId) {
                    setSelectedAction(
                        point.actions[0].actionUuid,
                        point.actions[0].actionName
                    );
                }
            }
        } else {
            clearSelectedAction();
        }
    }, [clearSelectedAction, getPointByUuid, selectedAction.actionId, selectedEventData, selectedProduct, setSelectedAction,]);

    const handleRenameAction = (newName: string) => {
        if (!selectedAction.actionId) return;
        updateAction(selectedAction.actionId, { actionName: newName });

        if (selectedPointData) {
            const updatedActions = selectedPointData.actions.map((action) =>
                action.actionUuid === selectedAction.actionId ? { ...action, actionName: newName } : action
            );
            setSelectedPointData({
                ...selectedPointData,
                actions: updatedActions,
            });
        }
    };

    const handleSpeedChange = (value: string) => {
        if (!selectedEventData) return;
        updateEvent(selectedProduct.productId, selectedEventData.data.modelUuid, {
            speed: parseFloat(value),
        });
    };

    const handlePickPointChange = (value: string) => {
        if (!selectedAction.actionId || !selectedPointData) return;
        const [x, y, z] = value.split(",").map(Number);

        updateAction(selectedAction.actionId, {
            process: {
                startPoint: [x, y, z] as [number, number, number],
                endPoint: selectedPointData.actions.find((a) => a.actionUuid === selectedAction.actionId)?.process.endPoint || null,
            },
        });
    };

    const handlePlacePointChange = (value: string) => {
        if (!selectedAction.actionId || !selectedPointData) return;
        const [x, y, z] = value.split(",").map(Number);

        updateAction(selectedAction.actionId, {
            process: {
                startPoint: selectedPointData.actions.find((a) => a.actionUuid === selectedAction.actionId)?.process.startPoint || null,
                endPoint: [x, y, z] as [number, number, number],
            },
        });
    };

    const availableActions = {
        defaultOption: "pickAndPlace",
        options: ["pickAndPlace"],
    };

    const currentSpeed = selectedEventData?.data.type === "roboticArm"
        ? selectedEventData.data.speed.toString()
        : "0.5";

    const currentAction = selectedPointData?.actions.find((a) => a.actionUuid === selectedAction.actionId);

    const currentPickPoint = currentAction?.process.startPoint
        ? `${currentAction.process.startPoint[0]},${currentAction.process.startPoint[1]},${currentAction.process.startPoint[2]}`
        : "";
    const currentPlacePoint = currentAction?.process.endPoint
        ? `${currentAction.process.endPoint[0]},${currentAction.process.endPoint[1]},${currentAction.process.endPoint[2]}`
        : "";

    return (
        <>
            <div className="global-props section">
                <div className="property-list-container">
                    <div className="property-item">
                        <InputWithDropDown
                            label="Speed"
                            value={currentSpeed}
                            min={0}
                            step={0.1}
                            defaultValue={"0.5"}
                            max={10}
                            activeOption="m/s"
                            onClick={() => { }}
                            onChange={handleSpeedChange}
                        />
                    </div>
                </div>
            </div>
            <section>
                <ActionsList
                    setSelectedPointData={setSelectedPointData}
                    selectedPointData={selectedPointData}
                    multipleAction
                />

                {selectedAction.actionId && currentAction && (
                    <div className="selected-actions-details">
                        <div className="selected-actions-header">
                            <RenameInput
                                value={selectedAction.actionName}
                                onRename={handleRenameAction}
                            />
                        </div>
                        <div className="selected-actions-list">
                            <LabledDropdown
                                defaultOption={activeOption}
                                options={availableActions.options}
                                onSelect={() => { }}
                                disabled={true}
                            />
                            <PickAndPlaceAction
                                pickPointValue={currentPickPoint}
                                pickPointOnChange={handlePickPointChange}
                                placePointValue={currentPlacePoint}
                                placePointOnChange={handlePlacePointChange}
                            />
                        </div>
                        <div className="tirgger">
                            <Trigger />
                        </div>
                    </div>
                )}
            </section>
        </>
    );
}

export default RoboticArmMechanics;
